#include "fintrf.h"
C     ========================================
C     Driver routine for DSBRDD (SBR toolbox)
C     ========================================

C     Gateway routine
      subroutine mexFunction(nlhs, plhs, nrhs, prhs)

      implicit none

C      EXTERNAL DSBRDT, DSBRDD
C      EXTERNAL ATOAB

C     Declarations
C     mexFunction arguments:
      mwPointer plhs(*), prhs(*)
      integer*8 nlhs, nrhs
      character*250 str

C     Function declarations:
      mwPointer mxGetPr, mxGetIr, mxGetJc
      mwPointer mxCreateDoubleMatrix
      mwPointer mxGetM, mxGetN

C     Pointers to input/output mxArrays:
      mwPointer in_ptr, out_ptr
      mwPointer pr, ir, jc
C     Array information:
      mwPointer mrows, ncols
      mwSize UM

C     Parameters for DSBRDT
      INTEGER*8 N, LDA, LDU, NB, LWORK, ISTEP, B, NSTEPS
      INTEGER*4 B0
      INTEGER INFO
      DOUBLE PRECISION DRPTOL, t0, t1
      INTEGER*8 ZERO, ONE
      DOUBLE PRECISION, ALLOCATABLE :: WORK(:) 
      DOUBLE PRECISION, ALLOCATABLE :: AB(:,:)
C      mwPointer AB
      mwPointer D, E, U

C     Statements
      ZERO = 0
      ONE = 1
      UM = 1

C     reading input data
      in_ptr = mxGetPr(prhs(1))
      call mxCopyPtrToInteger4(in_ptr, B0, UM)
      B = B0

C     Get the size of the input array.
      ncols = mxGetN(prhs(2))
      mrows = mxGetM(prhs(2))
            
      N = ncols
       
      LDA = 2*B
      LDU = N 
      
      NB = 0
      NSTEPS = 1

      LWORK = B*N
      
      DRPTOL = 1.0D-10

C     Create Fortran array from the input argument.
C      AB = mxGetPr(prhs(2))
      U  = mxGetPr(prhs(3))
      pr = mxGetPr(prhs(2))
      ir = mxGetIr(prhs(2))
      jc = mxGetJc(prhs(2))

      ALLOCATE(AB(LDA,N))

C     Converting to LAPACK/SBR band storage
      CALL AtoAB(N,LDA,%VAL(pr),%VAL(ir),%VAL(jc),AB,B)

C     Create matrix for the return arguments
      plhs(1) = mxCreateDoubleMatrix(ncols, ONE, 0.0)   
      plhs(2) = mxCreateDoubleMatrix(ncols-1, ONE, 0.0)
      plhs(3) = mxCreateDoubleMatrix(ncols, ncols, 0.0)

      D = mxgetpr(plhs(1))
      E = mxgetpr(plhs(2))

      ALLOCATE(WORK(LWORK))
      
C     calling SBR toolbox routine
      t0 = 0.0D0
      t1 = 0.0D0
      CALL cpu_time(t0)

      CALL DSBRDD('U', 'L', N, B, ONE, AB, LDA, DRPTOL,
     >                     %val(D), %val(E), %val(U), LDU, 
     >                     NSTEPS, ONE, NB,
     >                     WORK, LWORK, ISTEP, INFO )
  
      CALL cpu_time(t1) 

      DEALLOCATE(WORK)
      DEALLOCATE(AB)

C     Load the data into out_ptr, which is the output to MATLAB.
      out_ptr = mxGetPr(plhs(3))
      call mxCopyReal8ToPtr(%val(U),out_ptr,ncols*ncols) 
      
      return
      end

C     END of subroutine
