function [T, Q, info] = lapack_dsbtrd(A,kd,Q,opt)
% 
% This function uses LAPACK's routine DSBTRD
% for reducing a symmetric band matrix to tridiagonal form.
%
% References: Schwartz(1968), Kaufmann(1984), Bischof(1994)
%
% Dependencies: LAPACK, SBR toolbox
%
% input: A sparse symmetric band matrix, kd: bandwidth, 
%        opt=0: LAPACK DSBTRD, opt=1: SBR DSBRDD
%
% output: T tridiagonal sparse matrix, Q such that Q'*T*Q = A, info: flag
%
% (2015) Douglas Goncalves
%

    kd = int64(kd);
    n = length(A);
    info=1;


    %%%%%% MEX file caller for LAPACK routine %%%%%%%%%%%%%%%%%
    if (opt==0)
        clear('lapack_sbr');

        [td, toff, Q] = lapack_sbr(A, Q, double(kd)); % Q will be overwritten

        % other subdiagonal
        toff2 = [0; toff];
        toff  = [toff; 0];

        % sparse tridiagonal output matrix
        T = spdiags([toff td toff2], -1:1, n, n);
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %%%%%% MEX file caller for SBR routine%%%%%%%%%%%%%%%%%%%%%%
    if (opt==1)
        clear('dsbrdd')
        clear('dsbrdt')

        [td, toff, Q] = dsbrdt(kd, A, Q); % Q will be overwritten   

        % other subdiagonal
        toff2 = [0; toff];
        toff  = [toff; 0];

        % sparse tridiagonal output matrix
        T = spdiags([toff td toff2], -1:1, n, n);
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
end

