#include "mex.h"
#include "lapack.h"
#include "time.h"

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    long m, n, lwork, info=0;
    double *A, *work, *d;

    /* verify input/output arguments */
    if (nrhs != 1) {
        mexErrMsgTxt("One input argument required.");
    }
    if (nlhs > 2) {
        mexErrMsgTxt("Too many output arguments.");
    }
    if (!mxIsDouble(prhs[0]) || mxIsComplex(prhs[0])) {
        mexErrMsgTxt("Input matrix must be real double matrix.");
    }
    if ( mxIsSparse(prhs[0]) ) {
        mexErrMsgTxt("Input matrix must not be sparse.");
    }
    
    /* duplicate input matrix (since its contents will be overwritten) */
    plhs[0] = mxDuplicateArray(prhs[0]);
    
    /* dimensions of input matrix */
    m = mxGetM(plhs[0]);
    n = mxGetN(plhs[0]);

    /* create output matrices */
    plhs[1] = mxCreateDoubleMatrix(n, 1, mxREAL);

    /* get data from pointers */
    A = mxGetPr(plhs[0]);
    d = mxGetPr(plhs[1]);

    /* allocating space for working arrays */ 
    lwork = -1;
    info = 0;
    /* getting the optimal value for lwork */
    work = (double *)mxMalloc(sizeof(double));
    dsyev("V", "U", &n, A, &n, d, work, &lwork, &info );
    lwork = work[0];
    /* reallocating work */
    mxFree(work);
    work = (double *)mxMalloc(lwork*sizeof(double));
    /* calling LAPACK routine */
    dsyev("V", "U", &n, A, &n, d, work, &lwork, &info );
    
    mexPrintf("Done, info: %d \n", info);   
    
    /* cleanup */
    mxFree(work);

    /* check if call was successful */
    if (info < 0) {
        mexPrintf("\n Info: %d \n", info);
        mexErrMsgTxt("Illegal values in arguments.");
    } else if (info > 0) {
        mexErrMsgTxt("Failed to converge.");
    }
}
