#include "mex.h"
#include "lapack.h"
#include "time.h"

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    long m, n, kd, info=0;
    double *AB, *work, *td, *toff, *Q;
    double workopt = 0;
    int kmin, kmax, i, j, k, ix;
    mxArray *in1;
    mxArray *in2;
    double *pr;
    mwIndex *ir, *jc;
    
    clock_t begin, end;
    double time_spent;

    /* verify input/output arguments */
    if (nrhs != 3) {
        mexErrMsgTxt("Three input arguments required.");
    }
    if (nlhs > 3) {
        mexErrMsgTxt("Too many output arguments.");
    }
    if (!mxIsDouble(prhs[0]) || mxIsComplex(prhs[0])) {
        mexErrMsgTxt("Input matrix must be real double matrix.");
    }

    /* duplicate input matrix (since its contents will be overwritten) */
    in1 = mxDuplicateArray(prhs[0]);
    in2 = mxDuplicateArray(prhs[1]);
    
    /* dimensions of input matrix */
    m = mxGetM(in1);
    n = mxGetN(in1);
    //kd = m-1;
    kd = (long)(*mxGetPr(prhs[2]));
    m = kd + 1; // lda of AB
    
    mexPrintf("Input parameters:  kd: %ld, n: %ld, m: %ld \n", kd, n, m);

    /* create output matrices */
    plhs[0] = mxCreateDoubleMatrix(n, 1, mxREAL);
    plhs[1] = mxCreateDoubleMatrix(n-1, 1, mxREAL);

    /* get data from pointers */
    pr = mxGetPr(prhs[0]);
    ir = mxGetIr(prhs[0]);
    jc = mxGetJc(prhs[0]);
    
    td = mxGetPr(plhs[0]);
    toff = mxGetPr(plhs[1]);
    Q = mxGetPr(in2);
    
    // allocating and initializing AB
    AB = (double*)mxMalloc((kd+1)*n*sizeof(double));
    for (i = 0; i < (kd+1)*n; i++)  AB[i] = 0.0;

    /* converting to the required format */
    for (j=0; j<n; j++) {
        kmin = jc[j];
        kmax = jc[j+1]-1;
        //mexPrintf("j = %d, kmin: %d, kmax: %d \n", j, kmin, kmax);
        for (k=kmin; k<=kmax; k++) {
            if ( ir[k] >= j) {
                ix = ir[k] - j;
                AB[j*m + ix] = pr[k];
            }
        }
    }

    /* allocate workspace */
    work = (double *) mxMalloc(n * sizeof(double));

    /* perform symmetric band reduction */
    /* Q is overwritten */
    begin = clock();
    dsbtrd("U", "L", &n, &kd, AB, &m, td, toff, Q, &n, work, &info);
    end = clock();
    time_spent = (double)(end - begin) / CLOCKS_PER_SEC;
    
    mexPrintf("\n Time: %f, info: %d \n", time_spent, info);
    
    plhs[2] = mxDuplicateArray(in2); // return Q
    
    /* cleanup */
    mxFree(work);
    mxFree(AB);
    mxDestroyArray(in1);
    mxDestroyArray(in2);

    /* check if call was successful */
    if (info < 0) {
        mexPrintf("\n Info: %d \n", info);
        mexErrMsgTxt("Illegal values in arguments.");
    } else if (info > 0) {
        mexErrMsgTxt("Failed to converge.");
    }
}
