function [B, T1, T, R, K] = test_sdpproj_gallery()

% fixed seed for random number generators
rng(17624);
 
% output
T1 = [];
T = [];
R = [];
B = []; 
K = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %     % MATLAB gallery
%      % discrete neumann problem
%      n = 60^2
%      A = gallery('neumann',n);

%     % discrete Poisson problem
%     m = 100;
%     n = m^2
%     A = gallery('poisson',m); 
%
    % Pentagonal Toeplitz
    n = 80^2
    A = gallery('toeppen', n);
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % symmetrization
    A = (A + A')/2;

    % print the estimated condition number of A
    condest(A)
%         
    % projection using EIG
    fprintf('EIG... \n');
    tic; 
    [V,D] = eigdecomp(A);
    Am1 = product(V,D);
    tempo1 = toc;
    T1 = [T1 tempo1];

%     % projection using DSYEV
%     fprintf('DSYEV... \n');
%     clear('dsyev');
%     tic; 
%     [V, ev] = lapack_dsyev(full(A));
%     D = spdiags(ev, 0, n, n);
%     Am1 = product(V,D);
%     tempo1 = toc;
%     T1 = [T1 tempo1];
%
    
    % projection using polynomial fixed point iteration
    fprintf('PFP...\n');
    tic; 
    [X, Q, flag, k] = sdpproj(A, 1, 1e-4, 100);
    [Am, P] = products(Q,X,A);
    tempo = toc;
    T = [T tempo];
    K = [K k];
    
    % distance between projections
    res1 = norm(Am1-Am,'fro');
    R = [R res1];

end

function [V,D] = eigdecomp(A)

    [V,D] = eig(full(A)); 

end

function [Am] = product(V,D)

    Am = V*max(D,0)*V';
    
end

function [Am, P] = products(Q,X,A)

    if nargout>1
        P = Q*X*Q';        
        Am = P*A;
    else
        Am = (Q*X)*((Q')*A);
    end

end