function [B, T1, T, R, K] = test_sdpproj_ufsparse()

% fixed seed for random number generators
rng(17624);
 
% output
T1 = [];
T = [];
R = [];
B = [];
K = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % UF test-suite (Univ. of Florida Suite Sparse)
%    load './ufsparse/power.mat'
%    load './ufsparse/cell2.mat'
    load './ufsparse/bcsstm38.mat'
%    load './ufsparse/ted_B.mat'
    A = Problem.A; 
    n = length(A)
    % symmetrization
    A = (A + A')/2;
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

    % print the estimated condition number of A
    condest(A)
        
    % projection using EIG
    fprintf('EIG... \n');
    tic; 
    [V,D] = eigdecomp(A);
    Am1 = product(V,D);
    tempo1 = toc;
    T1 = [T1 tempo1];

%     % projection using DSYEV
%     fprintf('DSYEV... \n');
%     clear('dsyev');
%     tic; 
%     [V, ev] = lapack_dsyev(full(A));
%     D = spdiags(ev, 0, n, n);
%     Am1 = product(V,D);
%     tempo1 = toc;
%     T1 = [T1 tempo1];
    
    % projection using polynomial fixed point iteration
    fprintf('PFP...\n');
    tic; 
    [X, Q, flag, k] = sdpproj(A, 1, 1e-4, 100);
    [Am, P] = products(Q,X,A);
    tempo = toc;
    T = [T tempo];
    K = [K k];
    
    % distance between projections
    res1 = norm(Am1-Am,'fro');
    R = [R res1];

end

function [V,D] = eigdecomp(A)

    [V,D] = eig(full(A)); 

end

function [Am] = product(V,D)

    Am = V*max(D,0)*V';
    
end

function [Am, P] = products(Q,X,A)

    if nargout>1
        P = Q*X*Q';        
        Am = P*A;
    else
        Am = (Q*X)*((Q')*A);
    end

end